
#ifndef _MAT_H_
#define _MAT_H_


#include <cv.h>


#include<assert.h> 

#include <iostream>
using namespace std;




namespace cvppmat{

//*********************
//C++ Wrapper for CvMat
//*********************

template<class T>
class Mat{
private:
	CvMat*Matrix;

	//Alloc Data
	inline void AllocData(int Rows,int Cols,int Type){this->Matrix=cvCreateMat(Rows,Cols,Type);}

public:
	Mat(){Matrix=NULL; cout<<"ConstructorWithOut Parameters"<<endl;}
	explicit Mat(int Rows,int Cols=1);
	Mat(const Mat&a){ if(a.Matrix){ AllocData(a.Matrix->rows,a.Matrix->cols,a.Matrix->type); cvCopy(a.Matrix,Matrix);}
					else Matrix=NULL; cout<<"CopyConstructor"<<endl;}

	~Mat(){ ReleaseData(); cout<<"Destructor"<<endl;};	


	//Release Matrix Data
	void ReleaseData(){ if(Matrix) cvReleaseMat(&this->Matrix); Matrix=NULL;} 	

	//Return CvMat* Pointer 
	inline CvMat * ptr() {return Matrix;}

	inline int Type(){return Matrix->type;}
	inline int Rows(){return Matrix->rows;}
	inline int Cols(){return Matrix->cols;}
	inline int Size(){return Matrix->rows*Matrix->cols;}
	


	inline T& operator()(int i,int j){ return ((T*)(Matrix->data.ptr + Matrix->step*i))[j];}
	inline T& operator()(int i){ return ((T*)(Matrix->data.ptr))[i];}
	inline T& operator[](int i){ return ((T*)(Matrix->data.ptr))[i];}

	
	//Compatibility With OpenCV Functions
	operator CvMat* () const {return Matrix;}


	//overload Operator =
	inline Mat& operator=(const Mat&a){
		if(this != &a){
			ReleaseData();
			AllocData(a.Matrix->rows,a.Matrix->cols,a.Matrix->type);
			cvCopy(a.Matrix,Matrix);
		}
	// cout<<"operator="<<endl;
		return *this;
	}



	inline Mat& operator+=(const Mat&a){
		cvAdd(Matrix,a.Matrix,Matrix,NULL);
		return *this;
	}

	inline Mat& operator+=(const T&val){
		cvAddS(Matrix, cvScalar(val,val,val,val),Matrix,NULL);
		return *this;
	}

	inline Mat& operator-=(const Mat&a){
		cvSub(Matrix,a.Matrix,Matrix,NULL);
		return *this;
	}

	inline Mat& operator-=(const T&val){
		cvSubS(Matrix, cvScalar(val,val,val,val),Matrix,NULL);
		return *this;
	}

	inline Mat& operator*=(const Mat&a){
		Mat Op;
		Op=*this;
		this->ReleaseData();
		this->AllocData(Op.Matrix->rows,a.Matrix->cols,Op.Matrix->type);
		//cout<<Op; cout<<*this;
		cvGEMM(Op.Matrix,a.Matrix,1.0,NULL,0.0,this->Matrix,0);
		return *this;
	}

	inline Mat& operator*=(const T&val){
		CvMat*Zeros=cvCreateMat(Matrix->rows,Matrix->cols,Matrix->type); cvSetZero(Zeros);	
		cvScaleAdd(Matrix, cvScalar(val,val,val,val),Zeros,Matrix);
		cvReleaseMat(&Zeros);
		return *this;
	}



	inline Mat& operator/=(const Mat&a){
		cvDiv(Matrix,a.Matrix,Matrix,1);
		return *this;
	}

	inline Mat& operator/=(const T&val){
		CvMat*ConstVal=cvCreateMat(Matrix->rows,Matrix->cols,Matrix->type); 
		cvSet(ConstVal,cvScalar(val,val,val,val),NULL);	
		cvDiv(Matrix,ConstVal,Matrix,1);
		cvReleaseMat(&ConstVal);
		return *this;
	}


	inline bool IsEmpty(){ if(Matrix==NULL) return true; return false;}


	//Get Sub Matrix
	inline Mat operator() (int LowRow,int HighRow, int LowCol,int HighCol){
		int Rows=HighRow-LowRow, Cols=HighCol-LowCol;
		assert(Rows>0 && Cols>0);
	
		Mat SubMatrix(Rows,Cols);
		for(int i=0;i<Rows;i++)
			for(int j=0;j<Cols;j++) SubMatrix(i,j)=this->operator()(i+LowRow,j+LowCol);
		return SubMatrix;
	}

	//Get Row
	inline Mat GetRow(int Row){
		assert(Row>=0 && Row<Rows());
		Mat RowMat(1,Cols());		
		for(int i=0;i<Cols();i++) RowMat(0,i)=this->operator()(Row,i);
		return RowMat;
	}


	//Get Col
	inline Mat GetCol(int Col){
		assert(Col>=0 && Col<Cols());
		Mat ColMat(Rows(),1);		
		for(int i=0;i<Rows();i++) ColMat(i,0)=this->operator()(i,Col);
		return ColMat;
	}


	void Display(){
		if(Matrix){
			for(int i=0;i<Rows();i++){
				for(int j=0;j<Cols();j++) cout<<((T*)(this->Matrix->data.ptr + this->Matrix->step*i))[j]<<' ';
				cout<<endl;
			}
		}
	}
};


typedef Mat<short int> Matsi;
typedef Mat<int> Mati;
typedef Mat<float> Matf;
typedef Mat<double> Matd;



//Constructors
template<> inline Mat<short int>::Mat(int Rows,int Cols){ assert(Rows>0); assert(Cols>0); AllocData(Rows,Cols,CV_16SC1); cout<<"Constructor"<<endl;}
template<> inline Mat<int>::Mat(int Rows,int Cols){ assert(Rows>0); assert(Cols>0); AllocData(Rows,Cols,CV_32SC1); cout<<"Constructor"<<endl;}
template<> inline Mat<float>::Mat(int Rows,int Cols){ assert(Rows>0); assert(Cols>0); AllocData(Rows,Cols,CV_32FC1); cout<<"Constructor"<<endl;}
template<> inline Mat<double>::Mat(int Rows,int Cols){ assert(Rows>0); assert(Cols>0); AllocData(Rows,Cols,CV_64FC1); cout<<"Constructor"<<endl;}





template <class T>
inline ostream &operator<<(ostream &out,Mat<T> &m){
	if(!m.IsEmpty()){
		for(int i=0;i<m.Rows();i++){	
			for(int j=0;j<m.Cols();j++) out<<m(i,j)<<' ';
			out<<endl;	
		}
	}
	return out;
}


#define Operator(op) \
template<class T> inline Mat<T> operator op (const Mat<T>&a, const Mat<T>&b) { Mat<T> c(a); c op##=b; return c; } \
template<class T> inline Mat<T> operator op (const Mat<T>&a, const T&val) {Mat<T> c(a); c op##=val; return c;} \
template<class T> inline Mat<T> operator op (const T&val,const Mat<T>&a) {Mat<T> c(a); c op##=val; return c;} 

Operator(+)
Operator(-)
Operator(*)
#undef Operator


#define Operator(op) \
template<class T> inline Mat<T> operator op (const Mat<T>&a,const Mat<T>&b) { Mat<T> c(a); c op##=b; return c; } \
template<class T> inline Mat<T> operator op (const Mat<T>&a,const T&val) {Mat<T> c(a); c op##=val; return c;} \

Operator(/)
#undef Operator



//Overload ~ Operator: returns matrix transponse  
template <class T> inline Mat<T> operator~(Mat<T>&a){
	Mat<T> Transpose(a.Cols(),a.Rows()); 
	cvTranspose(a,Transpose); 
	return Transpose;
}








//*****************
//Related Functions
//*****************

//Returns Zeros Matrix
template <class T> inline Mat<T> zeros(int Rows,int Cols=1){ 
	Mat<T> Zeros(Rows,Cols);
	cvSetZero(Zeros);
	return Zeros;
}

//Returns Ones Matrix
template <class T> inline Mat<T> ones(int Rows,int Cols=1){ 
	Mat<T> Ones(Rows,Cols);
	cvSet(Ones,cvScalar(1,1,1,1));
	return Ones;
}


//Retuns (Scaled) Identity Matrix 
template <class T> inline Mat<T> eye(int Dim,const T&value=1){ 
	Mat<T> Eye(Dim,Dim);
	cvSetIdentity(Eye,cvRealScalar(value));
	return Eye;
}


//Retuns Dot Product 
template <class T> inline double dot(const Mat<T>&a,const Mat<T>&b){ 
	return cvDotProduct(a,b);
}

//Retuns Cross Product 
template <class T> inline Mat<T> cross(Mat<T>&a,Mat<T>&b){ 
	Mat<T> CrossProduct(a.Rows(),a.Cols());	
	cvCrossProduct(a,b,CrossProduct);
	return CrossProduct;
}


//Retuns Matrix Trace
template <class T> inline double trace(const Mat<T>&m){ 
	CvScalar TraceVal;
	TraceVal=cvTrace(m);
	return TraceVal.val[0];
}


//Retuns Determinant 
template <class T> inline double det(const Mat<T>&m){ 
	return cvDet(m);
}






//Inverse of Square Matrix
template <class T> inline Mat<T> inv(Mat<T>&m){
	Mat<T> Inv(m.Rows(),m.Cols());
	cvInvert(m,Inv,CV_LU);
	return Inv;
}

//Pseudo Inverse
template <class T> inline Mat<T> pinv(Mat<T>&m){
	Mat<T> Pinv(m.Cols(),m.Rows());
	cvInvert(m,Pinv,CV_SVD);
	return Pinv;
}


//SVD Decomposition: Return Only Singular values
template <class T> inline Mat<T> svd(Mat<T>&A){
	Mat<T> S(A.Cols()); 
	cvSVD(A,S);
	return S;
}


//Full SVD Decomposition
template <class T> inline void svd(Mat<T>&A,Mat<T>&U,Mat<T>&S,Mat<T>&V){
	U=Mat<T>(A.Rows(),A.Cols());
	S=Mat<T>(A.Cols());
	V=Mat<T>(A.Cols(),A.Cols());
	cvSVD(A,S,U,V,0);
}







}//end namespace cvppmat



//******
//Macros
//******

#define clear(m) cvSetZero(m)
 
#define zeros zeros<float>
#define zerosd zeros<double>
#define zerosi zeros<int>

#define ones ones<float>
#define onesd ones<double>
#define onesi ones<int>


#define eye eye<float>
#define eyed eye<double>
#define eyei eye<int>



#endif
